@@ -1,7 +1,7 @@
 //! Provides default values for the configuration.
 
 use crate::config::{
-    BlacklistConfig, BlacklistMode, Config, LoggingConfig, RouteConfig, RouteType,
+    BlacklistConfig, BlacklistMode, Config, HostConfig, LoggingConfig, RouteConfig, RouteType,
 };
 use crate::server::logger::LogLevel;
 
@@ -14,7 +14,7 @@ impl Default for Config {
             #[cfg(feature = "tls")]
             tls_config: None,
             default_websocket_proxy: None,
-            hosts: vec![Default::default()],
+            hosts: Vec::new(),
             default_host: Default::default(),
             #[cfg(feature = "plugins")]
             plugins: Vec::new(),
@@ -25,6 +25,15 @@ impl Default for Config {
     }
 }
 
+impl Default for HostConfig {
+    fn default() -> Self {
+        Self {
+            matches: "*".into(),
+            routes: vec![Default::default()],
+        }
+    }
+}
+
 impl Default for RouteConfig {
     fn default() -> Self {
         Self {
